from bs4 import BeautifulSoup
import re

def parse_releases_main_page(html_data):
    tdada = BeautifulSoup(html_data,features="html5lib").table.table
    data = []

    for row in tdada("tr"):
        if len(row('td')) != 8:
            continue

        columns = row('td')
        data.append({
            'url': columns[0].a.get('href'),
            'name': columns[0].a.get('href').replace('/project/',''),
            'description': columns[0].a.string,
            'actual_versions': [{
                'url':v.get('href'),
                'name':v.string,
                'date':d.string.strip(),
                } for v,d in zip(columns[1]('a'),columns[2]('span'))
            ],
            'plan_versions': [{
                'url':v.get('href'),
                'date':d.string.strip(),
                'update':ud.string.strip(),
                } for v,d,ud in zip(columns[3]('a'),columns[4]('span'),columns[5]('span'))
            ],
            'beta_versions': [{
                'url':v.get('href'),
                'name':v.string,
                'date':d.string.strip(),
                } for v,d in zip(columns[6]('a'),columns[7]('span'))
            ],
        })
    
    return data

def parse_project_page(html_data):

    tdada = BeautifulSoup(html_data,features="html5lib").table.table

    data = []

    for row in tdada.tbody('tr'):
        columns = row('td')
        data.append({
            'url': columns[0].a.get('href'),
            'version': columns[0].a.string.strip(),
            'date':columns[1].string.strip(),
            'previous_versions': columns[2].string.strip(),
        })

    return data

def parse_version_files(html_data):

    tdada = BeautifulSoup(html_data,features="html5lib")

    pattern = r"nick=(\S+)&ver=(\d+(?:.\d+)+)(?:&path=.*%5c(.*))?"

    data = []

    for row in tdada.table('tr')[1].div('a')[1:-1]:
        matcher = re.search(pattern,row.get('href'))
        nick = matcher.group(1)
        nick_wo_numbers = re.sub("\d",'',nick)
        version = matcher.group(2)
        filename = matcher.group(3)
        if (filename):
            data.append({
                'url': row.get('href'),
                'name': row.string.strip(),
                'nick': nick,
                'version': version,
                'file': filename,
                'filetype': filename
                    .replace('_'+version.replace('.','_'),'')
                    .replace('_'+version,'')
                    .replace(nick+'_','')
                    .replace(nick_wo_numbers+'_',''),
            })

    return data

def parse_sources(html_data):

    tdada = BeautifulSoup(html_data,features="html5lib")

    data = []

    for row in tdada('div',{"class": "downloadDist"})[0]('a'):
        data.append(row.get('href'))

    return data


from releases1c.request1c import Request1C
from releases1c.secrets import credentials
import os
from releases1c.progress import printProgressBar

releasesURL = "https://releases.1c.ru"

def get_data(url,flush_cookie=False):
    r1c = Request1C(**credentials,flush_cookie=flush_cookie)
    return r1c.get(url)

def get_head(url, flush_cookie=False):
    r1c = Request1C(**credentials,flush_cookie=flush_cookie)
    return r1c.get_head(url)

def get_stream(url, flush_cookie=False):
    r1c = Request1C(**credentials,flush_cookie=flush_cookie)
    return r1c.get_stream(url)

class WrongFiletype(Exception):
    filetype = ''
    matches = []
    def __init__(cls,fyletype,matches=[]):
        super()
        cls.fyletype = fyletype
        cls.matches = matches

def get_download_url_by_filetype(**kwargs):

    __file_list = [x for x in get_data(releasesURL+"/version_files?nick={product}&ver={version}".format(**kwargs),kwargs.get("flush_cookie")) if x.get('filetype')==kwargs.get("filetype")]
    if len(__file_list) == 0:
        raise WrongFiletype(kwargs.get("filetype"))
    elif len(__file_list) == 1:
        url = f"{releasesURL}{__file_list[0].get('url')}"
    else:
        raise WrongFiletype(kwargs.get("filetype"),__file_list)
    
    return url

def __is_dirname(path):
    if (path == '.' or path == '..' or os.path.isdir(path)):
        path = os.path.join(path,'')
    return path[-1] == os.sep

def get_filename_from_url(url):
    resph = get_head(url)

    if (resph.headers.get('content-disposition')):
        filename = [x[1].replace('"','') for x in [k.strip().split('=') for k in resph.headers.get('content-disposition').split(';')] if x[0] == 'filename'][0]
    else:
        filename = resph.url.split('/')[-1]
    return filename

def get_local_filename(url,path):
    if __is_dirname(path):
        filename = get_filename_from_url(url)
        return os.path.join(path,filename)
    else:
        return path

def download(**kwargs):
    url = get_download_url_by_filetype(**kwargs)
    mirrors = get_data(url, kwargs.get('flush_cookie'))
    #print('mirrors',mirrors)
    result = None
    for download_url in mirrors:
        result = download_file(download_url,kwargs.get('dest'))
        if result:
            break
    return result

def download_file(url:str, dest:str):
    file_name = get_local_filename(url,dest)
    print('Downloading to:', file_name)
    r = get_stream(url)
    with open(file_name, "wb") as f:
        total_length = r.headers.get('content-length')
        if total_length is None: # no content length header
            f.write(r.content)
            print('Complete!')
        else:
            dl = 0
            total_length = int(total_length)
            for chunk in r.iter_content(chunk_size=int(total_length/5000)):
                if chunk:
                    f.write(chunk)
                    f.flush()
                dl += len(chunk)
                printProgressBar(dl, total_length, prefix = 'Progress:', suffix = f'Complete ({int(total_length/1024/1024)} Mb)', length = 50)
    
    return {'result':True, 'file_name':file_name}



import requests, pickle
import json
import base64
import os
from urllib.parse import urlparse
from releases1c import parsing
import sys
from clint.textui import progress
import time
from releases1c.progress import printProgressBar

class Request1C:
    
    session = None
    loginURL = "https://login.1c.ru"
    login = ""
    password = ""
    __saved_cookies_file = "__cookies.bin"
    
    def __init__(self, login:str, password:str, flush_cookie:bool):
        self.session = requests.Session()
        # load cookies
        if os.path.isfile(self.__saved_cookies_file) and not flush_cookie:
            #print('load cookies from file',self.__saved_cookies_file)
            with open(self.__saved_cookies_file, 'rb') as f:
                self.session.cookies.update(pickle.load(f))
        self.login = login
        self.password = password

    def get_url_params(self,url:str):
        url_parts = urlparse(url)
        url_query_params = list(map(lambda x: x.split('=')[0],url_parts.query.split('&')))
        url_path = url_parts.path.split('/')[1]
        can_be_file = False
        
        if (url_path == 'total'):
            parse_op = parsing.parse_releases_main_page
        elif (url_path == 'project'):
            parse_op = parsing.parse_project_page
        elif (url_path == 'version_files'):
            if (set(['nick','ver']) == set(url_query_params)):
                parse_op = parsing.parse_version_files
            else:
                print('>> UNSUPPORTED >>',url_path,url_query_params)
        elif (url_path == 'version_file'):
            if (set(['nick','ver','path']) == set(url_query_params)):
                parse_op = parsing.parse_sources
                # OR
                can_be_file = True
            else:
                print('>> UNSUPPORTED >>',url_path,url_query_params)

        else:
            print('>> UNSUPPORTED >>',url_path,url_query_params)
        
        return {'parse_op':parse_op,'can_be_file':can_be_file}
    
    def get_head(self, url):
        resph = self.session.head(url,allow_redirects=True)
        return resph
    
    def get_stream(self, url):
        return self.session.get(url, stream=True)

    def get(self, url:str):
        config_url = self.get_url_params(url)
        
        if (config_url['can_be_file']):
            resph = self.session.head(url,allow_redirects=True)
            if resph.url.startswith(self.loginURL):
                resph = self.authenticate(url,only_headers=True)
            
            # The way, how we decide is this a file ot not
            if not (resph.headers.get('X-Application-Context')): # This is a file
                return [resph.url]

        resp = self.session.get(url)

        if resp.url.startswith(self.loginURL):
            resp = self.authenticate(url)

        return config_url['parse_op'](resp.content.decode())
    
    def authenticate(self, url, only_headers=False):

        # obtain token
        auth_data = {"login":self.login,"password":self.password,"serviceNick": url}
        payload = json.dumps(auth_data)
        resp = self.session.post(f"{self.loginURL}/rest/public/ticket/get",data=payload, headers={'Content-Type':'application/json'})
        if resp.status_code != 200:
            print(resp.status_code,resp.headers,resp.url, resp.json())
            return resp

        # login
        if (only_headers):
            resp = self.session.head("{loginURL}/ticket/auth?token={ticket}".format(loginURL=self.loginURL,**(resp.json())),headers={'Authentication':'Basic {}'.format(base64.b64encode("{}:{}".format(self.login,self.password).encode()).decode())},allow_redirects=True)
        else:
            resp = self.session.get("{loginURL}/ticket/auth?token={ticket}".format(loginURL=self.loginURL,**(resp.json())),headers={'Authentication':'Basic {}'.format(base64.b64encode("{}:{}".format(self.login,self.password).encode()).decode())})
        
        if resp.status_code != 200:
            print(resp.status_code,resp.headers,resp.url)
            return resp
        
        # save cookies to file
        with open(self.__saved_cookies_file, 'wb') as f:
            pickle.dump(self.session.cookies, f)
        
        return resp



import click
from releases1c import __version__
from releases1c.request1c import Request1C
from releases1c.secrets import credentials
from releases1c.downloader import get_download_url_by_filetype, WrongFiletype, download
from releases1c import parsing
import os

releasesURL = "https://releases.1c.ru"

def safe_list_get (l, idx, default):
    try:
        return l[idx]
    except IndexError:
        return default

@click.group()
def cli():
    pass

@cli.command(short_help='Get information about projects, versions and files')
@click.option('-f','--flush-cookie',type=bool,is_flag=True,default=False, show_default=True,help="Flush the cookie")
@click.argument('parameters',nargs=-1,metavar='[PROJECT] | [PROJECT VERSION] | [PROJECT VERSION FILETYPE]')
def info(parameters,flush_cookie) -> None:
    '''Get information for project and verison and filetype'''

    settings = prepare_executors(parameters,flush_cookie)

    #print('>> DEBUG >>',settings['url'](parameters))

    print_info(settings, flush_cookie)


def print_info(settings, flush_cookie):
    
    data = get_data(settings['url'],flush_cookie)

    print('Getting data from URL:',settings['url'])
    
    #print(data)
    for row in data:
        settings['display'](row)

def prepare_executors(parameters,flush_cookie):

    lpr = len(parameters)
    is_file_list = False

    if lpr == 0:
        url = f"{releasesURL}/total"
        display = lambda product: print('-',product.get('name'),'-',product.get('description'),[v['name'] for v in product.get('actual_versions')])
    elif lpr == 1:
        url = f"{releasesURL}/project/{parameters[0]}?allUpdates=true"
        display = lambda v: print('-',v.get('version'),'-',v.get('date'),v.get('previous_versions'))
    elif lpr == 2:
        url = f"{releasesURL}/version_files?nick={parameters[0]}&ver={parameters[1]}"
        display = lambda f: print('-',f['filetype'],'-',f['file'],'-',f['name'])
    elif lpr == 3:
        try:
            url = get_download_url_by_filetype(**dict(zip(['product','version','filetype'],parameters)))
        except WrongFiletype as err:
            print("Wrong filetype:",parameters[2])
            return prepare_executors(parameters[:2],flush_cookie)
        else:

            #print('url',url)
            #url = f"{releasesURL}{__file_list[0].get('url')}"
            display = lambda f: print('-',f)
            is_file_list = True
    else:
        print("Unsupported parameters:", parameters[2:])
    
    return {'url':url, 'display':display, 'is_file_list':is_file_list}

def get_data(url,flush_cookie):
    r1c = Request1C(**credentials,flush_cookie=flush_cookie)
    return r1c.get(url)

@cli.command(name='download')
@click.argument('parameters',nargs=3,metavar='PROJECT VERSION FILETYPE')
@click.argument('filepath',type=click.Path(exists=False,writable=True))
@click.pass_context
def download_cmd(ctx,parameters,filepath):
    '''Download file for project and version'''
    #print('filepath',filepath,os.path.isfile(filepath),os.path.isdir(filepath))
    try:
        download(**credentials,**dict(zip(['product','version','filetype'],parameters)),dest=filepath)
    except WrongFiletype as err:
        print("Wrong filetype:",parameters[2])
        ctx.invoke(info(parameters[:2]))

@cli.command()
def version():
    print('Version:',__version__)

if __name__ == "__main__":
    cli(auto_envvar_prefix='RELEASES1C')